package my.petstore.entity;

import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import org.springframework.beans.support.PagedListHolder;

public class Cart {

    private final Map<Integer,CartItem> itemMap = new ConcurrentHashMap<Integer,CartItem>();

    private final PagedListHolder<CartItem> itemList = new PagedListHolder<CartItem>();

    public Cart() {
        this.itemList.setPageSize(4);
    }

    // XXX would be a defensive copy
    public List<CartItem> getAllCartItems() {
        return itemList.getSource();
    }

    public PagedListHolder<CartItem> getCartItemList() {
        return itemList;
    }

    public int getNumberOfItems() {
        return itemList.getSource().size();
    }

    public boolean containsItemId(Integer itemId) {
        return itemMap.containsKey(itemId);
    }

    public void addItem(Item item, boolean isInStock) {
        CartItem cartItem = itemMap.get(item.getId());
        if (cartItem == null) {
            cartItem = new CartItem();
            cartItem.setItem(item);
            cartItem.setQuantity(0);
            cartItem.setInStock(isInStock);
            itemMap.put(item.getId(), cartItem);
            itemList.getSource().add(cartItem);
        }
        cartItem.incrementQuantity();
    }


    public Item removeItemById(Integer itemId) {
        CartItem cartItem = itemMap.remove(itemId);
        if (cartItem == null) {
            return null;
        } else {
            itemList.getSource().remove(cartItem);
            return cartItem.getItem();
        }
    }

    public void incrementQuantityByItemId(Integer itemId) {
        CartItem cartItem = itemMap.get(itemId);
        cartItem.incrementQuantity();
    }

    public void setQuantityByItemId(Integer itemId, int quantity) {
        CartItem cartItem = itemMap.get(itemId);
        cartItem.setQuantity(quantity);
    }

    public int getTotal() {
        int total = 0;
        for (CartItem cartItem : getAllCartItems()) {
            Item item = cartItem.getItem();
            int listPrice = item.getListPrice();
            int quantity = cartItem.getQuantity();
            total += listPrice * quantity;
        }
        return total;
    }
}
