package my.petstore.service;

import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import my.petstore.entity.Cart;
import my.petstore.entity.CartItem;
import my.petstore.entity.Item;

import org.springframework.beans.factory.annotation.Autowired;

public class CartServiceImpl implements CartService {
    @Autowired
    private ProductService productService;

    public void addItemToCart(Cart cart, Integer workingItemId) {
        if (cart.containsItemId(workingItemId)) {
            cart.incrementQuantityByItemId(workingItemId);
        } else {
            // TODO
            // isInStock is a "real-time" property that must be updated
            // every time an item is added to the cart, even if other
            // item details are cached.
            boolean isInStock = productService.isItemInStock(workingItemId);
            Item item = productService.getItem(workingItemId);
            cart.addItem(item, isInStock);
        }
    }

    public void updateCartQuantities(Cart cart, Map<Integer,Integer> quantities) {
        Set<Integer> removeIds = new HashSet<Integer>();
        for (CartItem cartItem : cart.getAllCartItems()) {
            Integer itemId = cartItem.getItem().getId();
            int quantity = quantities.get(itemId);
            cart.setQuantityByItemId(itemId, quantity);
            if (quantity < 1) {
                removeIds.add(itemId);
            }
        }
        for (Integer id : removeIds) {
            cart.removeItemById(id);
        }
    }
}
