package my.petstore.service;

import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;

import my.petstore.entity.Inventory;
import my.petstore.entity.Item;
import my.petstore.entity.LineItem;
import my.petstore.entity.OrderData;

import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

@Repository
@Transactional
public class OrderServiceImpl implements OrderService {

    @PersistenceContext
    private EntityManager em;

    public void storeOrderData(OrderData orderData) {
        for (LineItem lineItem : orderData.getLineItems()) {
            int decrement = lineItem.getQuantity();
            Inventory inventory = this.em.find(Item.class, lineItem.getItem().getId()).getInventory();
            int current = inventory.decrementQuantity(decrement);
            if (current < 0) {
                ;// TODO out of order error
            }
            this.em.merge(inventory);
        }

        OrderData merged = this.em.merge(orderData);
        orderData.setId(merged.getId());
        this.em.flush();
    }


    @Transactional(readOnly = true)
    public OrderData getOrderData(Integer orderId) {
        OrderData orderData = this.em.find(OrderData.class, orderId);
        return orderData != null ? orderData : OrderData.NULL;
    }

    @Transactional(readOnly = true)
    public List<OrderData> getOrderData() {
        return this.em.createQuery("SELECT orderData FROM OrderData orderData").getResultList();
    }
}
