package my.petstore.web;

import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import my.petstore.entity.Cart;
import my.petstore.service.CartService;
import my.petstore.service.ProductService;
import my.petstore.util.PagedListUtil;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.util.WebUtils;

@Controller
public class CartController {

    @Autowired
    private ProductService productService;

    @Autowired
    private CartService cartService;

    @RequestMapping(value="/cart", method=RequestMethod.GET)
    public ModelAndView handleRequest(@RequestParam(defaultValue="") String page, HttpSession session) {
        UserSession userSession = (UserSession) session.getAttribute("userSession");
        if (userSession != null) {
            PagedListUtil.movePage(userSession.getMyList(), page);
        }
        Cart cart = (Cart) WebUtils.getOrCreateSessionAttribute(session, "sessionCart", Cart.class);
        return new ModelAndView("Cart", "cart", cart);
    }

    @RequestMapping(value="/checkout", method=RequestMethod.GET)
    public ModelAndView checkout(@RequestParam(defaultValue="") String page, HttpSession session) {
        Cart cart = (Cart) WebUtils.getOrCreateSessionAttribute(session, "sessionCart", Cart.class);
        PagedListUtil.movePage(cart.getCartItemList(), page);
        return new ModelAndView("Checkout", "cart", cart);
    }

    // XXX
    // Cart sync? => it's taken care of by Cart's ConcurrentHashMap

    // XXX HTTP method should not be GET
    @RequestMapping(value="/addItemToCart", method=RequestMethod.GET)
    public ModelAndView addItemToCart(@RequestParam(required=true) Integer workingItemId, HttpSession session) {
        Cart cart = (Cart) WebUtils.getOrCreateSessionAttribute(session, "sessionCart", Cart.class);
        cartService.addItemToCart(cart, workingItemId);
        return new ModelAndView("Cart", "cart", cart);
    }

    @RequestMapping(value="/updateCartQuantities", method=RequestMethod.POST)
    public ModelAndView updateCartQuantities(HttpServletRequest request, HttpSession session) {
        Cart cart = (Cart) WebUtils.getOrCreateSessionAttribute(session, "sessionCart", Cart.class);
        Map<Integer, Integer> quantities = getQuantities(request);
        cartService.updateCartQuantities(cart, quantities);
        return new ModelAndView("Cart", "cart", cart);
    }

    private Map<Integer,Integer> getQuantities(HttpServletRequest request) {
        Map<Integer, Integer> quantities = new HashMap<Integer,Integer>();
        Map<String,String[]> params = request.getParameterMap();
        for (Map.Entry<String,String[]> param : params.entrySet()) {
            try {
                int id = Integer.parseInt(param.getKey());
                quantities.put(id, Integer.parseInt(param.getValue()[0]));
            } catch (NumberFormatException ex) {
                ;// ignore on purpose
            }
        }
        return quantities;
    }

    // XXX HTTP method should not be GET
    @RequestMapping(value="/removeItemFromCart", method=RequestMethod.GET)
    public ModelAndView removeItemFromCart(@RequestParam Integer workingItemId, HttpSession session) {
        Cart cart = (Cart) WebUtils.getOrCreateSessionAttribute(session, "sessionCart", Cart.class);
        cart.removeItemById(workingItemId);
        return new ModelAndView("Cart", "cart", cart);
    }
}
