package my.petstore.web;

import javax.servlet.http.HttpSession;

import my.petstore.service.ProductService;
import my.petstore.util.PagedListUtil;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.support.PagedListHolder;
import org.springframework.stereotype.Controller;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.servlet.ModelAndView;

@Controller
public class SearchProductsController {

    @Autowired
    private ProductService productService;

    // TODO it would be better to delegate to SearchService
    @RequestMapping(value="/search", method=RequestMethod.GET)
    public ModelAndView handleRequest(@RequestParam(required=false) String keyword, @RequestParam(required=false) String page, HttpSession session) {
        PagedListHolder productList;
        if (keyword != null) {
            if (!StringUtils.hasLength(keyword)) {
                return new ModelAndView("Error", "message", "Please enter a keyword to search for, then press the search button.");
            }

            productList = new PagedListHolder(productService.searchProductList(keyword.toLowerCase()));
            productList.setPageSize(4);
            session.setAttribute("SearchProductsController_productList", productList);
        } else {
            productList = (PagedListHolder) session.getAttribute("SearchProductsController_productList");
            if (productList == null) {
                return new ModelAndView("Error", "message", "Your session has timed out. Please start over again.");
            }
            PagedListUtil.movePage(productList, page);
        }
        return new ModelAndView("SearchProducts", "productList", productList);
    }
}
